@@ -12,6 +12,7 @@
 
 #include <KConfigGroup>
 #include <KWindowSystem>
+#include <KX11Extras>
 
 #include "applet.h"
 #include "appletquickitem.h"
@@ -57,13 +58,17 @@ private:
 
 using namespace PlasmaQuick;
 
-AppletPopup::AppletPopup(QWindow *parent)
-    : PopupPlasmaWindow(parent)
+AppletPopup::AppletPopup()
+    : PopupPlasmaWindow()
 {
     setAnimated(true);
     setFlags(flags() | Qt::Dialog);
 
-    PlasmaShellWaylandIntegration::get(this)->setRole(QtWayland::org_kde_plasma_surface::role::role_appletpopup);
+    if (KWindowSystem::isPlatformX11()) {
+        KX11Extras::setType(winId(), NET::AppletPopup);
+    } else {
+        PlasmaShellWaylandIntegration::get(this)->setRole(QtWayland::org_kde_plasma_surface::role::role_appletpopup);
+    }
 
     auto edgeForwarder = new EdgeEventForwarder(this);
     edgeForwarder->setMargins(padding());
@@ -96,6 +101,17 @@ AppletPopup::AppletPopup(QWindow *parent)
     connect(this, &PlasmaWindow::paddingChanged, this, &AppletPopup::updateMaxSize);
     connect(this, &PlasmaWindow::paddingChanged, this, &AppletPopup::updateSize);
     connect(this, &PlasmaWindow::paddingChanged, this, &AppletPopup::updateMinSize);
+
+    connect(this, &PlasmaWindow::screenChanged, this, [this](QScreen *screen) {
+        if (m_oldScreen) {
+            disconnect(m_oldScreen, &QScreen::geometryChanged, this, &AppletPopup::updateMaxSize);
+        }
+        if (screen) {
+            connect(screen, &QScreen::geometryChanged, this, &AppletPopup::updateMaxSize);
+        }
+        m_oldScreen = screen;
+        updateMaxSize();
+    });
 }
 
 AppletPopup::~AppletPopup()
@@ -221,7 +237,12 @@ void AppletPopup::updateMaxSize()
     if (!m_layoutChangedProxy) {
         return;
     }
-    setMaximumSize(m_layoutChangedProxy->maximumSize().grownBy(padding()));
+    QSize size = m_layoutChangedProxy->maximumSize().grownBy(padding());
+    if (screen()) {
+        size.setWidth(std::min(size.width(), int(std::round(screen()->geometry().width() * 0.95))));
+        size.setHeight(std::min(size.height(), int(std::round(screen()->geometry().height() * 0.95))));
+    }
+    setMaximumSize(size);
 }
 
 void AppletPopup::updateSize()
