@@ -237,7 +237,7 @@ void DialogPrivate::updateTheme()
         KWindowEffects::enableBlurBehind(q, false);
         KWindowEffects::enableBackgroundContrast(q, false);
         q->setMask(QRegion());
-        DialogShadows::self()->removeWindow(q);
+        DialogShadows::instance()->removeWindow(q);
     } else {
         auto prefix = QStringLiteral("");
         if ((backgroundHints & Dialog::SolidBackground) == Dialog::SolidBackground) {
@@ -269,7 +269,7 @@ void DialogPrivate::updateTheme()
             q->setMask(dialogBackground->mask());
         }
         if (q->isVisible()) {
-            DialogShadows::self()->addWindow(q, dialogBackground->enabledBorders());
+            DialogShadows::instance()->addWindow(q, dialogBackground->enabledBorders());
         }
     }
     updateInputShape();
@@ -571,7 +571,10 @@ void DialogPrivate::updateLayoutParameters()
 
     dialogBackground->setSize(QSizeF(q->width(), q->height()));
 
-    repositionIfOffScreen();
+    if (!needsSetupNextExpose && visible) {
+        // Only reposition after successful setup; otherwise repositionIfOffScreen will override the default position set by kwin under wayland
+        repositionIfOffScreen();
+    }
     updateTheme();
 
     // setting the minimum or maximum size will resize the window instantly and min <= max is enforced
@@ -1008,7 +1011,6 @@ void Dialog::setMainItem(QQuickItem *mainItem)
 
         if (mainItem) {
             mainItem->setParentItem(contentItem());
-            mainItem->setFlag(QQuickItem::ItemIsFocusScope, true);
 
             connect(mainItem, SIGNAL(widthChanged()), this, SLOT(slotMainItemSizeChanged()));
             connect(mainItem, SIGNAL(heightChanged()), this, SLOT(slotMainItemSizeChanged()));
@@ -1386,7 +1388,7 @@ void Dialog::showEvent(QShowEvent *event)
     d->updateSizeFromAppletInterface();
 
     if (d->backgroundHints != Dialog::NoBackground) {
-        DialogShadows::self()->addWindow(this, d->dialogBackground->enabledBorders());
+        DialogShadows::instance()->addWindow(this, d->dialogBackground->enabledBorders());
     }
 
     if (KWindowSystem::isPlatformX11()) {
@@ -1413,10 +1415,10 @@ bool Dialog::event(QEvent *event)
         // sometimes non null regions arrive even for non visible windows
         // for which surface creation would fail
         if (d->needsSetupNextExpose && isVisible()) {
-            d->needsSetupNextExpose = false;
             d->updateVisibility(true);
             const bool ret = QQuickWindow::event(event);
             d->updateTheme();
+            d->needsSetupNextExpose = false;
             return ret;
         }
     } else if (event->type() == QEvent::Show) {
